@@ -181,7 +181,7 @@ struct winbindd_request {
                            character is. */	
 			fstring user;
 			fstring pass;
-		        fstring required_membership_sid;
+		        fstring require_membership_of_sid;
 		} auth;              /* pam_winbind auth module */
                 struct {
                         unsigned char chal[8];
@@ -192,7 +192,7 @@ struct winbindd_request {
                         fstring nt_resp;
                         uint16 nt_resp_len;
 			fstring workstation;
-		        fstring required_membership_sid;
+		        fstring require_membership_of_sid;
                 } auth_crap;
                 struct {
                     fstring user;
