@@ -39,6 +39,8 @@ static const struct table_node archi_table[]= {
 	{"Windows NT R4000",     "W32MIPS",	2 },
 	{"Windows NT Alpha_AXP", "W32ALPHA",	2 },
 	{"Windows NT PowerPC",   "W32PPC",	2 },
+	{"Windows IA64",         "IA64",        3 },
+	{"Windows x64",          "x64",         3 },
 	{NULL,                   "",		-1 }
 };
 
@@ -296,6 +298,18 @@ static void display_print_info_3(PRINTER_INFO_3 *i3)
 	printf("\n");
 }
 
+/****************************************************************************
+printer info level 7 display function
+****************************************************************************/
+static void display_print_info_7(PRINTER_INFO_7 *i7)
+{
+	fstring guid = "";
+	rpcstr_pull(guid, i7->guid.buffer,sizeof(guid), -1, STR_TERMINATE);
+	printf("\tguid:[%s]\n", guid);
+	printf("\taction:[0x%x]\n", i7->action);
+}
+
+
 /* Enumerate printers */
 
 static WERROR cmd_spoolss_enum_printers(struct cli_state *cli, 
@@ -314,7 +328,7 @@ static WERROR cmd_spoolss_enum_printers(struct cli_state *cli,
 		return WERR_OK;
 	}
 
-	if (argc == 2)
+	if (argc >= 2)
 		info_level = atoi(argv[1]);
 
 	if (argc == 3)
@@ -434,7 +448,7 @@ static WERROR cmd_spoolss_enum_ports(struct cli_state *cli,
 	WERROR         		result;
 	uint32                  needed, info_level = 1;
 	PORT_INFO_CTR 		ctr;
-	int 			returned;
+	uint32 			returned;
 	
 	if (argc > 2) {
 		printf("Usage: %s [level]\n", argv[0]);
@@ -507,7 +521,7 @@ static WERROR cmd_spoolss_setprinter(struct cli_state *cli,
 
 	slprintf(servername, sizeof(servername)-1, "\\\\%s", cli->desthost);
 	strupper_m(servername);
-	fstrcpy(printername, argv[1]);
+	slprintf(printername, sizeof(servername)-1, "%s\\%s", servername, argv[1]);
 	fstrcpy(user, cli->user_name);
 
 	/* get a printer handle */
@@ -546,6 +560,76 @@ static WERROR cmd_spoolss_setprinter(struct cli_state *cli,
 	return result;
 }
 
+/***********************************************************************
+ * Set printer name - use a level2 set.
+ */
+static WERROR cmd_spoolss_setprintername(struct cli_state *cli,
+                                       TALLOC_CTX *mem_ctx,
+                                       int argc, const char **argv)
+{
+	POLICY_HND 	pol;
+	WERROR		result;
+	uint32 		needed;
+	uint32 		info_level = 2;
+	BOOL 		opened_hnd = False;
+	PRINTER_INFO_CTR ctr;
+	fstring 	printername,
+			servername,
+			user,
+			new_printername;
+
+	if (argc == 1 || argc > 3) {
+		printf("Usage: %s printername new_printername\n", argv[0]);
+
+		return WERR_OK;
+	}
+
+	/* Open a printer handle */
+	if (argc == 3) {
+		fstrcpy(new_printername, argv[2]);
+	}
+
+	slprintf(servername, sizeof(servername)-1, "\\\\%s", cli->desthost);
+	strupper_m(servername);
+	slprintf(printername, sizeof(printername)-1, "%s\\%s", servername, argv[1]);
+	fstrcpy(user, cli->user_name);
+
+	/* get a printer handle */
+	result = cli_spoolss_open_printer_ex(cli, mem_ctx, printername, "", 
+				PRINTER_ALL_ACCESS, servername,
+				user, &pol);
+				
+	if (!W_ERROR_IS_OK(result))
+		goto done;
+
+	opened_hnd = True;
+
+	/* Get printer info */
+        result = cli_spoolss_getprinter(cli, mem_ctx, 0, &needed, &pol, info_level, &ctr);
+
+        if (W_ERROR_V(result) == ERRinsufficientbuffer)
+                result = cli_spoolss_getprinter(cli, mem_ctx, needed, NULL, &pol, info_level, &ctr);
+
+        if (!W_ERROR_IS_OK(result))
+                goto done;
+
+
+	/* Modify the printername. */
+	init_unistr(&ctr.printers_2->printername, new_printername);
+	ctr.printers_2->devmode = NULL;
+	ctr.printers_2->secdesc = NULL;
+
+	result = cli_spoolss_setprinter(cli, mem_ctx, &pol, info_level, &ctr, 0);
+	if (W_ERROR_IS_OK(result))
+		printf("Success in setting printername.\n");
+
+ done:
+	if (opened_hnd)
+		cli_spoolss_close_printer(cli, mem_ctx, &pol);
+
+	return result;
+}
+
 /***********************************************************************
  * Get printer information
  */
@@ -616,6 +700,9 @@ static WERROR cmd_spoolss_getprinter(struct cli_state *cli,
 	case 3:
 		display_print_info_3(ctr.printers_3);
 		break;
+	case 7:
+		display_print_info_7(ctr.printers_7);
+		break;
 	default:
 		printf("unknown info level %d\n", info_level);
 		break;
@@ -949,7 +1036,7 @@ static WERROR cmd_spoolss_getdriver(struct cli_state *cli,
 	slprintf(servername, sizeof(servername)-1, "\\\\%s", cli->desthost);
 	strupper_m(servername);
 	fstrcpy(user, cli->user_name);
-	fstrcpy(printername, argv[1]);
+	slprintf(printername, sizeof(servername)-1, "%s\\%s", servername, argv[1]);
 	if (argc == 3)
 		info_level = atoi(argv[2]);
 
@@ -1056,6 +1143,13 @@ static WERROR cmd_spoolss_enum_drivers(struct cli_state *cli,
 				cli, mem_ctx, needed, NULL, info_level, 
 				archi_table[i].long_archi, &returned, &ctr);
 
+		if (W_ERROR_V(werror) == W_ERROR_V(WERR_INVALID_ENVIRONMENT)) {
+			printf ("Server does not support environment [%s]\n", 
+				archi_table[i].long_archi);
+			werror = WERR_OK;
+			continue;
+		}
+
 		if (returned == 0)
 			continue;
 			
@@ -1239,7 +1333,7 @@ static BOOL init_drv_info_3_members (
 	/* allocate the space; add one extra slot for a terminating NULL.
 	   Each filename is NULL terminated and the end contains a double
 	   NULL */
-	if ((info->dependentfiles=(uint16*)talloc(mem_ctx, (len+1)*sizeof(uint16))) == NULL)
+	if ((info->dependentfiles=TALLOC_ARRAY(mem_ctx, uint16, len+1)) == NULL)
 	{
 		DEBUG(0,("init_drv_info_3_members: Unable to malloc memory for dependenfiles\n"));
 		return False;
@@ -1451,6 +1545,61 @@ done:
 }
 
 
+static WERROR cmd_spoolss_deletedriverex(struct cli_state *cli, 
+                                         TALLOC_CTX *mem_ctx,
+                                         int argc, const char **argv)
+{
+	WERROR result, ret = WERR_UNKNOWN_PRINTER_DRIVER;
+ 
+	int   i;
+	int vers = -1;
+ 
+	const char *arch = NULL;
+ 
+	/* parse the command arguements */
+	if (argc < 2 || argc > 4) {
+		printf ("Usage: %s <driver> [arch] [version]\n", argv[0]);
+		return WERR_OK;
+	}
+
+	if (argc >= 3)
+		arch = argv[2];
+	if (argc == 4)
+		vers = atoi (argv[3]);
+ 
+ 
+	/* delete the driver for all architectures */
+	for (i=0; archi_table[i].long_archi; i++) {
+
+		if (arch &&  !strequal( archi_table[i].long_archi, arch)) 
+			continue;
+
+		if (vers >= 0 && archi_table[i].version != vers)
+			continue;
+
+		/* make the call to remove the driver */
+		result = cli_spoolss_deleteprinterdriverex(
+			cli, mem_ctx, archi_table[i].long_archi, argv[1], archi_table[i].version); 
+
+		if ( !W_ERROR_IS_OK(result) ) 
+		{
+			if ( !W_ERROR_EQUAL(result, WERR_UNKNOWN_PRINTER_DRIVER) ) {
+				printf ("Failed to remove driver %s for arch [%s] (version: %d): %s\n", 
+					argv[1], archi_table[i].long_archi, archi_table[i].version, dos_errstr(result));
+			}
+		} 
+		else 
+		{
+			printf ("Driver %s and files removed for arch [%s] (version: %d).\n", argv[1], 
+			archi_table[i].long_archi, archi_table[i].version);
+			ret = WERR_OK;
+		}
+	}
+  
+	return ret;
+}
+
+
 static WERROR cmd_spoolss_deletedriver(struct cli_state *cli, 
                                          TALLOC_CTX *mem_ctx,
                                          int argc, const char **argv)
@@ -1844,6 +1993,7 @@ static WERROR cmd_spoolss_setprinterdata(struct cli_state *cli,
 	PRINTER_INFO_CTR ctr;
 	PRINTER_INFO_0 info;
 	REGISTRY_VALUE value;
+	UNISTR2 data;
 
 	/* parse the command arguements */
 	if (argc != 4) {
@@ -1881,10 +2031,11 @@ static WERROR cmd_spoolss_setprinterdata(struct cli_state *cli,
 
 	/* Set the printer data */
 	
+	init_unistr2(&data, argv[3], UNI_STR_TERMINATE);
 	fstrcpy(value.valuename, argv[2]);
 	value.type = REG_SZ;
-	value.size = strlen(argv[3]) + 1;
-	value.data_p = talloc_memdup(mem_ctx, argv[3], value.size);
+	value.size = data.uni_str_len * 2;
+	value.data_p = TALLOC_MEMDUP(mem_ctx, data.buffer, value.size);
 
 	result = cli_spoolss_setprinterdata(cli, mem_ctx, &pol, &value);
 		
@@ -2266,8 +2417,7 @@ static WERROR cmd_spoolss_rffpcnex(struct cli_state *cli,
 	option.option_type_ptr = 1;
 	option.count = option.ctr.count = 2;
 
-	option.ctr.type = (SPOOL_NOTIFY_OPTION_TYPE *)talloc(
-		mem_ctx, sizeof(SPOOL_NOTIFY_OPTION_TYPE) * 2);
+	option.ctr.type = TALLOC_ARRAY(mem_ctx, SPOOL_NOTIFY_OPTION_TYPE, 2);
 
 	ZERO_STRUCT(option.ctr.type[0]);
 	option.ctr.type[0].type = PRINTER_NOTIFY_TYPE;
@@ -2307,13 +2457,14 @@ struct cmd_set spoolss_commands[] = {
 	{ "SPOOLSS"  },
 
 	{ "adddriver",		RPC_RTYPE_WERROR, NULL, cmd_spoolss_addprinterdriver,	PI_SPOOLSS, "Add a print driver",                  "" },
-	{ "addprinter",	RPC_RTYPE_WERROR, NULL, cmd_spoolss_addprinterex,	PI_SPOOLSS, "Add a printer",                       "" },
+	{ "addprinter",		RPC_RTYPE_WERROR, NULL, cmd_spoolss_addprinterex,	PI_SPOOLSS, "Add a printer",                       "" },
 	{ "deldriver",		RPC_RTYPE_WERROR, NULL, cmd_spoolss_deletedriver,	PI_SPOOLSS, "Delete a printer driver",             "" },
+	{ "deldriverex",	RPC_RTYPE_WERROR, NULL, cmd_spoolss_deletedriverex,	PI_SPOOLSS, "Delete a printer driver with files",  "" },
 	{ "enumdata",		RPC_RTYPE_WERROR, NULL, cmd_spoolss_enum_data,		PI_SPOOLSS, "Enumerate printer data",              "" },
-	{ "enumdataex",	RPC_RTYPE_WERROR, NULL, cmd_spoolss_enum_data_ex,	PI_SPOOLSS, "Enumerate printer data for a key",    "" },
+	{ "enumdataex",		RPC_RTYPE_WERROR, NULL, cmd_spoolss_enum_data_ex,	PI_SPOOLSS, "Enumerate printer data for a key",    "" },
 	{ "enumkey",		RPC_RTYPE_WERROR, NULL, cmd_spoolss_enum_printerkey,	PI_SPOOLSS, "Enumerate printer keys",              "" },
 	{ "enumjobs",		RPC_RTYPE_WERROR, NULL, cmd_spoolss_enum_jobs,          PI_SPOOLSS, "Enumerate print jobs",                "" },
-	{ "enumports", 	RPC_RTYPE_WERROR, NULL, cmd_spoolss_enum_ports, 	PI_SPOOLSS, "Enumerate printer ports",             "" },
+	{ "enumports", 		RPC_RTYPE_WERROR, NULL, cmd_spoolss_enum_ports, 	PI_SPOOLSS, "Enumerate printer ports",             "" },
 	{ "enumdrivers", 	RPC_RTYPE_WERROR, NULL, cmd_spoolss_enum_drivers, 	PI_SPOOLSS, "Enumerate installed printer drivers", "" },
 	{ "enumprinters", 	RPC_RTYPE_WERROR, NULL, cmd_spoolss_enum_printers, 	PI_SPOOLSS, "Enumerate printers",                  "" },
 	{ "getdata",		RPC_RTYPE_WERROR, NULL, cmd_spoolss_getprinterdata,	PI_SPOOLSS, "Get print driver data",               "" },
@@ -2321,16 +2472,16 @@ struct cmd_set spoolss_commands[] = {
 	{ "getdriver",		RPC_RTYPE_WERROR, NULL, cmd_spoolss_getdriver,		PI_SPOOLSS, "Get print driver information",        "" },
 	{ "getdriverdir",	RPC_RTYPE_WERROR, NULL, cmd_spoolss_getdriverdir,	PI_SPOOLSS, "Get print driver upload directory",   "" },
 	{ "getprinter", 	RPC_RTYPE_WERROR, NULL, cmd_spoolss_getprinter, 	PI_SPOOLSS, "Get printer info",                    "" },
-	{ "getprintprocdir",RPC_RTYPE_WERROR, NULL, cmd_spoolss_getprintprocdir,    PI_SPOOLSS, "Get print processor directory",       "" },
 	{ "openprinter",	RPC_RTYPE_WERROR, NULL, cmd_spoolss_open_printer_ex,	PI_SPOOLSS, "Open printer handle",                 "" },
-	{ "setdriver", 	RPC_RTYPE_WERROR, NULL, cmd_spoolss_setdriver,		PI_SPOOLSS, "Set printer driver",                  "" },
+	{ "setdriver", 		RPC_RTYPE_WERROR, NULL, cmd_spoolss_setdriver,		PI_SPOOLSS, "Set printer driver",                  "" },
 	{ "getprintprocdir",	RPC_RTYPE_WERROR, NULL, cmd_spoolss_getprintprocdir,    PI_SPOOLSS, "Get print processor directory",       "" },
 	{ "addform",		RPC_RTYPE_WERROR, NULL, cmd_spoolss_addform,            PI_SPOOLSS, "Add form",                            "" },
 	{ "setform",		RPC_RTYPE_WERROR, NULL, cmd_spoolss_setform,            PI_SPOOLSS, "Set form",                            "" },
 	{ "getform",		RPC_RTYPE_WERROR, NULL, cmd_spoolss_getform,            PI_SPOOLSS, "Get form",                            "" },
-	{ "deleteform",	RPC_RTYPE_WERROR, NULL, cmd_spoolss_deleteform,         PI_SPOOLSS, "Delete form",                         "" },
+	{ "deleteform",		RPC_RTYPE_WERROR, NULL, cmd_spoolss_deleteform,         PI_SPOOLSS, "Delete form",                         "" },
 	{ "enumforms",		RPC_RTYPE_WERROR, NULL, cmd_spoolss_enum_forms,         PI_SPOOLSS, "Enumerate forms",                     "" },
-	{ "setprinter",	RPC_RTYPE_WERROR, NULL, cmd_spoolss_setprinter,         PI_SPOOLSS, "Set printer comment",                 "" },
+	{ "setprinter",		RPC_RTYPE_WERROR, NULL, cmd_spoolss_setprinter,         PI_SPOOLSS, "Set printer comment",                 "" },
+	{ "setprintername",	RPC_RTYPE_WERROR, NULL, cmd_spoolss_setprintername,	PI_SPOOLSS, "Set printername",                 "" },
 	{ "setprinterdata",	RPC_RTYPE_WERROR, NULL, cmd_spoolss_setprinterdata,     PI_SPOOLSS, "Set REG_SZ printer data",             "" },
 	{ "rffpcnex",		RPC_RTYPE_WERROR, NULL, cmd_spoolss_rffpcnex,           PI_SPOOLSS, "Rffpcnex test", "" },
 
