@@ -538,8 +538,6 @@ static NTSTATUS cmd_stat(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, c
 	printf("  Modify: %s", ctime(&(st.st_mtime)));
 	printf("  Change: %s", ctime(&(st.st_ctime)));
 
-	SAFE_FREE(pwd);
-	SAFE_FREE(grp);
 	return NT_STATUS_OK;
 }
 
@@ -606,8 +604,6 @@ static NTSTATUS cmd_fstat(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc,
 	printf("  Modify: %s", ctime(&(st.st_mtime)));
 	printf("  Change: %s", ctime(&(st.st_ctime)));
 
-	SAFE_FREE(pwd);
-	SAFE_FREE(grp);
 	return NT_STATUS_OK;
 }
 
@@ -662,8 +658,6 @@ static NTSTATUS cmd_lstat(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc,
 	printf("  Modify: %s", ctime(&(st.st_mtime)));
 	printf("  Change: %s", ctime(&(st.st_ctime)));
 	
-	SAFE_FREE(pwd);
-	SAFE_FREE(grp);
 	return NT_STATUS_OK;
 }
 
